/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#ifndef _NAVIGATOR_BAR_H_
#define _NAVIGATOR_BAR_H_

#include <QWidget>
#include <QToolBar>
#include <QPushButton>
#include <QIcon>
#include <QString>
#include <QEvent>
#include <QMouseEvent>
#include <QMenu>
#include <QTimer>

/*
********************************************************************************
*                                                                              *
*    Class NavigatorBar                                                        *
*                                                                              *
********************************************************************************
*/
class MenuModel;
class NavigatorItemButton;

class NavigatorBar : public QToolBar
{
Q_OBJECT
  public:
    NavigatorBar(QWidget *parent = 0);
    virtual void setModel(MenuModel * m);

    QSize sizeHint() const;

  public slots:
    void hideMenu();

  private:
    MenuModel                   *m_model;
    QList<NavigatorItemButton*>  m_listButton;

  private slots:
    void updateEntry();
};


/*
********************************************************************************
*                                                                              *
*    Class NavigatorItemButton                                                 *
*                                                                              *
********************************************************************************
*/
class NavigatorItemMenu;

class NavigatorItemButton : public QPushButton
{
Q_OBJECT
  public:
    NavigatorItemButton( const QIcon & icon, const QString & text, QWidget *parent );
    QSize sizeHint() const;

    NavigatorItemMenu* itemMenu() {return m_menu;}
    void setDefaultAction(QAction *a) {m_default_action = a;}

  protected:
    void paintEvent(QPaintEvent* event);
    void enterEvent(QEvent *e);
    void leaveEvent(QEvent *e);
    void mousePressEvent ( QMouseEvent * e );

  private slots:
    void slot_button_clicked();

  private:
    NavigatorItemMenu   *m_menu;
    QAction             *m_default_action;
    bool                 is_popup_mouse_over;

  signals :
    void menu_activated();
};

/*
********************************************************************************
*                                                                              *
*    Class NavigatorItemMenu                                                   *
*                                                                              *
********************************************************************************
*/
class NavigatorItemMenu : public QMenu
{
  public:
    NavigatorItemMenu(QWidget * parent = 0);

  protected:
    void enterEvent(QEvent *e);
    void leaveEvent(QEvent *e);
};


#endif // _NAVIGATOR_BAR_H_
