/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "settings_widget.h"
#include "ContextView/widget/context_graphicitem.h"
#include "globalshortcuts.h"

#include "utilities.h"
#include "yarocksettings.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class PageGeneral                                                         *
*                                                                              *
********************************************************************************
*/
PageGeneral::PageGeneral(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    //! create Gui
    createGui();
}


//! ----------- createGui ------------------------------------------------------
void PageGeneral::createGui()
{
    m_title   = new TitleGraphicItem();
    m_title->m_name   = tr("General");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);
    /*-------------------------------------------------*/
    /* General settings                                */
    /* ------------------------------------------------*/
    //! dbus & mpris parameter
    ui_check_enableMpris = new QCheckBox(main_widget);
    ui_check_enableMpris->setText(tr("Enable Mpris"));

    ui_groupbox_dbus = new QGroupBox(tr("Enable notification"));
    ui_groupbox_dbus->setCheckable(true);
    ui_groupbox_dbus->setChecked(false);
    ui_groupbox_dbus->setFlat(true);

    ui_check_vol_notif = new QCheckBox();
    ui_check_vol_notif->setText(tr("notify on volume change"));

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(ui_check_vol_notif);
    ui_groupbox_dbus->setLayout(vbox);

    //! systray parameter
    ui_check_systray = new QCheckBox(main_widget);
    ui_check_systray->setText(tr("Minimize application to systray"));

    ui_check_hideAtStartup = new QCheckBox(main_widget);
    ui_check_hideAtStartup->setText(tr("Hide window at startup"));

    //! menu ui control
    ui_check_enable_mouseover = new QCheckBox(main_widget);
    ui_check_enable_mouseover->setText(tr("Show menu on mouse over"));

    /*-------------------------------------------------*/
    /* Graphical Interface settings                    */
    /* ------------------------------------------------*/
    QLabel *lbl1 = new QLabel(tr("Mpris interface"), main_widget);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));

    QLabel *lbl2 = new QLabel(tr("Notifications"), main_widget);
    lbl2->setFont(QFont("Arial",10,QFont::Bold));

    QLabel *lbl3 = new QLabel(tr("Others"), main_widget);
    lbl3->setFont(QFont("Arial",10,QFont::Bold));

    //! layout
    vl0->addWidget(lbl1);
    vl0->addWidget(ui_check_enableMpris);
    vl0->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Expanding) );
    vl0->addWidget(lbl2);
    vl0->addWidget(ui_groupbox_dbus);
    vl0->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Expanding) );
    vl0->addWidget(lbl3);
    vl0->addWidget(ui_check_systray);
    vl0->addWidget(ui_check_hideAtStartup);
    vl0->addWidget(ui_check_enable_mouseover);

    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageGeneral::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PageGeneral::doLayout()
{
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}


QSize PageGeneral::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height() + 50
      );
}


//! ----------- saveSettings ---------------------------------------------------
void PageGeneral::saveSettings()
{
    Debug::debug() << "PageGeneral::saveSettings";
    SETTINGS()->_useTrayIcon         = this->ui_check_systray->isChecked();
    SETTINGS()->_hideAtStartup       = this->ui_check_hideAtStartup->isChecked();
    SETTINGS()->_useDbusNotification = this->ui_groupbox_dbus->isChecked();
    SETTINGS()->_useMpris            = this->ui_check_enableMpris->isChecked();
    SETTINGS()->_popup_on_mouseover  = this->ui_check_enable_mouseover->isChecked();
}

//! ----------- restoreSettings ------------------------------------------------
void PageGeneral::restoreSettings()
{
    this->ui_check_systray->setChecked( SETTINGS()->_useTrayIcon );
    this->ui_check_hideAtStartup->setChecked( SETTINGS()->_hideAtStartup );
    this->ui_groupbox_dbus->setChecked( SETTINGS()->_useDbusNotification );
    this->ui_check_enableMpris->setChecked( SETTINGS()->_useMpris );
    this->ui_check_enable_mouseover->setChecked( SETTINGS()->_popup_on_mouseover );
}


bool PageGeneral::isSystrayChanged()
{
    return SETTINGS()->_useTrayIcon != ui_check_systray->isChecked();
}

bool PageGeneral::isDbusChanged()
{
    return SETTINGS()->_useDbusNotification != ui_groupbox_dbus->isChecked();
}

bool PageGeneral::isMprisChanged()
{
    return SETTINGS()->_useMpris != ui_check_enableMpris->isChecked();
}






/*
********************************************************************************
*                                                                              *
*    Class PagePlayer                                                          *
*                                                                              *
********************************************************************************
*/
PagePlayer::PagePlayer(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    //! create Gui
    createGui();
}

//! ----------- createGui ------------------------------------------------------
void PagePlayer::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Player settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    /*-------------------------------------------------*/
    /* Player settings                                 */
    /* ------------------------------------------------*/
    ui_stopOnPlayqueueClear = new QCheckBox(main_widget);
    ui_stopOnPlayqueueClear->setText(tr("Stop playing on playqueue clear"));

    ui_restartPlayingAtStartup = new QCheckBox(main_widget);
    ui_restartPlayingAtStartup->setText(tr("Restart playing at startup"));

    ui_groupBox = new QGroupBox(tr("Use ReplayGain"));
    ui_groupBox->setCheckable(true);
    ui_groupBox->setChecked(false);
    ui_groupBox->setFlat(true);

    ui_mode_track = new QRadioButton(tr("mode radio"));
    ui_mode_album = new QRadioButton(tr("mode album"));

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(new QLabel(tr("Choose ReplayGain mode"), main_widget));
    vbox->addWidget(ui_mode_track);
    vbox->addWidget(ui_mode_album);
    ui_groupBox->setLayout(vbox);

    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);
    vl0->addWidget(ui_stopOnPlayqueueClear);
    vl0->addWidget(ui_restartPlayingAtStartup);
    vl0->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Expanding) );
    vl0->addWidget(ui_groupBox);

    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );


    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PagePlayer::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PagePlayer::doLayout()
{
    //Debug::debug() << "## PagePlayer::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}


QSize PagePlayer::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height() + 50
      );
}

void PagePlayer::saveSettings()
{
    Debug::debug() << "PagePlayer::saveSettings";

    SETTINGS()->_stopOnPlayqueueClear = this->ui_stopOnPlayqueueClear->isChecked();
    SETTINGS()->_restartPlayingAtStartup = this->ui_restartPlayingAtStartup->isChecked();

    if(!ui_groupBox->isChecked())
      SETTINGS()->_replaygain = 0;
    else if(ui_mode_track->isChecked())
      SETTINGS()->_replaygain = 1;
    else
      SETTINGS()->_replaygain = 2;
}

void PagePlayer::restoreSettings()
{
    this->ui_stopOnPlayqueueClear->setChecked( SETTINGS()->_stopOnPlayqueueClear );
    this->ui_restartPlayingAtStartup->setChecked( SETTINGS()->_restartPlayingAtStartup );

    this->ui_groupBox->setChecked( SETTINGS()->_replaygain != 0 );
    this->ui_mode_track->setChecked( SETTINGS()->_replaygain == 1 );
    this->ui_mode_album->setChecked( SETTINGS()->_replaygain == 2 );
}


/*
********************************************************************************
*                                                                              *
*    Class PageLibrary                                                         *
*                                                                              *
********************************************************************************
*/
PageLibrary::PageLibrary(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    //! create Gui
    createGui();

    //! connection
    connect(this->ui_add_path_button, SIGNAL(clicked()), this, SLOT(on_addPathButton_clicked()));
    connect(this->ui_remove_path_button, SIGNAL(clicked()), this, SLOT(on_removePathButton_clicked()));
    connect(this->ui_remove_all_button, SIGNAL(clicked()), this, SLOT(on_removeAllPathsButton_clicked()));

    connect(this->ui_auto_update, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam1()));
    connect(this->ui_search_cover, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam2()));

    connect(this->ui_choose_db, SIGNAL(currentIndexChanged(QString)), SLOT(loadDatabaseParam(QString)));
    connect(this->ui_db_new_button, SIGNAL(clicked()), this, SLOT(newDatabaseParam()));
    connect(this->ui_db_del_button, SIGNAL(clicked()), this, SLOT(delDatabaseParam()));
    connect(this->ui_db_rename_button, SIGNAL(clicked()), this, SLOT(renameDatabaseParam()));

    connect(this->ui_enable_multiDb, SIGNAL(clicked()), this, SLOT(enableMultiDatabase()));


    enableMultiDatabase();
}

//! ----------- createGui ------------------------------------------------------
void PageLibrary::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Library settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    /*-------------------------------------------------*/
    /* Libray settings                                 */
    /* ------------------------------------------------*/
    QVBoxLayout* verticalLayout = new QVBoxLayout(main_widget);

    //! label
    QLabel *lbl1 = new QLabel(tr("Database Name"), main_widget);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));

    //! Check box --> enable Multi Databases
    ui_enable_multiDb = new QCheckBox(main_widget);
    ui_enable_multiDb->setText(tr("Enable Multi Database support"));

    //! horizontal Layout (database list + save + delete)
    ui_choose_db = new QComboBox(main_widget);
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(ui_choose_db->sizePolicy().hasHeightForWidth());
    ui_choose_db->setSizePolicy(sizePolicy);

    ui_db_new_button = new QToolButton(main_widget);
    ui_db_new_button->setIcon(QIcon(":/images/document-new-48x48.png"));
    ui_db_new_button->setToolTip(tr("New database"));

    ui_db_del_button = new QToolButton(main_widget);
    ui_db_del_button->setIcon(QIcon::fromTheme("edit-delete"));
    ui_db_del_button->setToolTip(tr("Delete database"));

    ui_db_rename_button = new QToolButton(main_widget);
    ui_db_rename_button->setIcon(QIcon(":/images/rename-48x48.png"));
    ui_db_rename_button->setToolTip(tr("Rename database"));

    QHBoxLayout *hl1 = new QHBoxLayout();
    hl1->addWidget(ui_choose_db);
    hl1->addWidget(ui_db_new_button);
    hl1->addWidget(ui_db_del_button);
    hl1->addWidget(ui_db_rename_button);

    //! label
    QLabel *lbl2 = new QLabel(tr("Database Properties"), main_widget);
    lbl2->setFont(QFont("Arial",10,QFont::Bold));

    //! horizontal Layout (source path list + add + delete + delete all)
    ui_paths_list = new QListWidget(main_widget);
    ui_paths_list->setMinimumHeight(180);

    QHBoxLayout* hl2 = new QHBoxLayout();
    hl2->addWidget(ui_paths_list);

    //! vertical Layout (button Remove/Remove All/Add)
    ui_add_path_button = new QPushButton(main_widget);
    ui_add_path_button->setText(tr("Add ..."));

    ui_remove_path_button = new QPushButton(main_widget);
    ui_remove_path_button->setText(tr("Remove"));

    ui_remove_all_button = new QPushButton(main_widget);
    ui_remove_all_button->setText(tr("Remove all"));

    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(ui_add_path_button);
    vl2->addWidget(ui_remove_path_button);
    vl2->addWidget(ui_remove_all_button);
    vl2->addItem(new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding));
    hl2->addLayout(vl2);

    //! Check box auto check database
    ui_auto_update = new QCheckBox(main_widget);
    ui_auto_update->setText(tr("Update collections automatically after start"));

    //! Check box --> search cover into directory
    ui_search_cover = new QCheckBox(main_widget);
    ui_search_cover->setText(tr("Search cover art from file directory"));

    verticalLayout->addWidget(lbl1);
    verticalLayout->addWidget(ui_enable_multiDb);
    verticalLayout->addLayout(hl1);
    verticalLayout->addItem( new QSpacerItem(20, 30, QSizePolicy::Minimum, QSizePolicy::Fixed) );
    verticalLayout->addWidget(lbl2);
    verticalLayout->addLayout(hl2);
    verticalLayout->addWidget(ui_auto_update);
    verticalLayout->addWidget(ui_search_cover);


    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );


    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageLibrary::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageGeneral::resizeEvent";
    doLayout();
}

void PageLibrary::doLayout()
{
    //Debug::debug() << "## PageLibrary::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}

QSize PageLibrary::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height() + 50
      );
}


//! ---- restoreSettings -------------------------------------------------------
void PageLibrary::restoreSettings()
{
    //! read settings from DatabaseManager
    DatabaseManager *dbManager = DatabaseManager::instance();
    Debug::debug() << "PageLibrary::restoreSettings dbManager->multiDb" << dbManager->multiDb;

    m_db_params.clear();
    ui_choose_db->clear();

    ui_enable_multiDb->setChecked(dbManager->multiDb);
    enableMultiDatabase();
    //Debug::debug() << "SettingCollectionPage::restoreSettings dbManager->DB_NAME" << dbManager->DB_NAME;

    foreach (const QString& name, dbManager->DB_NAME_LIST()) {
      //Debug::debug() << "SettingCollectionPage::restoreSettings dbManager->params.keys()" << name;
      addDatabaseParam(name, dbManager->DB_PARAM(name));
    }

    int selectedIdx = ui_choose_db->findText(dbManager->DB_NAME);
    if (selectedIdx != -1)
      ui_choose_db->setCurrentIndex(selectedIdx);

    _isChanged = false;
}

//! ---- saveSettings -----------------------------------------------------------
void PageLibrary::saveSettings()
{
    Debug::debug() << "PageLibrary::saveSettings";

    //! save settings file
    DatabaseManager *dbManager = DatabaseManager::instance();

    dbManager->multiDb = ui_enable_multiDb->isChecked();
    dbManager->DB_NAME = ui_choose_db->currentText();
    dbManager->CLEAR_PARAM();

    foreach (const QString& name, m_db_params.keys()) {
      //Debug::debug() << "SettingCollectionPage::saveSettings m_db_params[name].autoRebuil" << m_db_params[name].autoRebuild;
      //Debug::debug() << "SettingCollectionPage::saveSettings m_db_params[name].checkCover" << m_db_params[name].checkCover;
      dbManager->SET_PARAM(name, m_db_params[name]);
    }

    dbManager->saveSettings();
}


//! ---- database instance management ------------------------------------------
void PageLibrary::enableMultiDatabase()
{
    //Debug::debug() << "SettingCollectionPage::enableMultiDatabase";
    ui_choose_db->setEnabled(ui_enable_multiDb->isChecked());
    ui_db_new_button->setEnabled(ui_enable_multiDb->isChecked());
    ui_db_del_button->setEnabled(ui_enable_multiDb->isChecked());
    ui_db_rename_button->setEnabled(ui_enable_multiDb->isChecked());
}


void PageLibrary::updateDatabaseParam1()
{
    QString dbName = ui_choose_db->currentText();

    if(m_db_params.contains(dbName)) {
      m_db_params[dbName].autoRebuild  = ui_auto_update->isChecked();
    }
}


void PageLibrary::updateDatabaseParam2()
{
    QString dbName = ui_choose_db->currentText();

    if(m_db_params.contains(dbName)) {
      m_db_params[dbName].checkCover   = ui_search_cover->isChecked();
    }
}


void PageLibrary::loadDatabaseParam(QString dbName)
{
   if(dbName.isEmpty()) return;

   if(m_db_params.contains(dbName)) {
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   NAME = " << dbName;
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   m_db_params[dbName].autoRebuild = " << m_db_params[dbName].autoRebuild;
     //Debug::debug() << "SettingCollectionPage::loadDatabaseParam   m_db_params[dbName].checkCover = " << m_db_params[dbName].checkCover;

     ui_auto_update->setChecked(m_db_params[dbName].autoRebuild);
     ui_search_cover->setChecked(m_db_params[dbName].checkCover);

     ui_paths_list->clear();
     foreach (QString path, m_db_params[dbName].sourcePathList) {
       ui_paths_list->addItem(path);
     }
      _isChanged = true;
    }
}


void PageLibrary::newDatabaseParam()
{
    QString currentDbName = ui_choose_db->currentText();

    QString name = QInputDialog::getText(m_parent, tr("New database"), tr("Name"),QLineEdit::Normal, currentDbName);

    if (name.isEmpty()) return;

    if (m_db_params.contains(name)) {
      QMessageBox::warning(
        m_parent, tr("New database"),
        tr("The database  \"%1\" already exist, please try another name").arg(name),
        QMessageBox::Ok);

      return;
    }

    DB::S_dbParam dbParam;
    dbParam.autoRebuild    = false;
    dbParam.checkCover     = true;
    dbParam.sourcePathList = QStringList();

    addDatabaseParam(name, dbParam);

    ui_choose_db->setCurrentIndex(ui_choose_db->findText(name));

    //! TODO ? create cover folder, playlist folder
}


void PageLibrary::addDatabaseParam(const QString& name, const DB::S_dbParam& dbParam)
{
    m_db_params[name].autoRebuild    = dbParam.autoRebuild;
    m_db_params[name].checkCover     = dbParam.checkCover;
    m_db_params[name].sourcePathList.clear();
    m_db_params[name].sourcePathList << dbParam.sourcePathList;

    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.sourcePathList" << dbParam.sourcePathList;
    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.autoRebuild" << dbParam.autoRebuild;
    //Debug::debug() << "SettingCollectionPage::addDatabaseParam  dbParam.checkCover" << dbParam.checkCover;

    if (ui_choose_db->findText(name) == -1)
      ui_choose_db->addItem(name);
}


void PageLibrary::delDatabaseParam()
{
    QString name = ui_choose_db->currentText();

    if (!m_db_params.contains(name) || name.isEmpty()) return;

    int returnCode = QMessageBox::question(
      m_parent, tr("Delete database properties"),
      tr("Are you sure you want to delete the \"%1\" database ?").arg(name),
      QMessageBox::Yes, QMessageBox::No);

    if (returnCode == QMessageBox::No) return;

    m_db_params.remove(name);
    ui_choose_db->removeItem(ui_choose_db->currentIndex());

    //! TODO delete database file + playlist folder + album folder
}

void PageLibrary::renameDatabaseParam()
{
    //! rename an existing database
    Debug::debug() << "SettingCollectionPage::renameDatabaseParam";

    QString oldName = ui_choose_db->currentText();

    QString newName = QInputDialog::getText(m_parent, tr("Rename database"), tr("Name"),QLineEdit::Normal, oldName);
    if (newName.isEmpty() || (newName == oldName) )
      return;

    if (m_db_params.contains(oldName))
    {
      Debug::debug() << "SettingCollectionPage::renameDatabaseParam";

      DB::S_dbParam dbParam = m_db_params.take(oldName);
      addDatabaseParam(newName, dbParam);
      ui_choose_db->removeItem(ui_choose_db->findText(oldName));
      //! TODO rename dabatase file, cover folder, playlist folder

      QCryptographicHash hash(QCryptographicHash::Sha1);
      hash.addData(oldName.toUtf8().constData());
      QFile oldDb(QString(UTIL::CONFIGDIR + "/" + hash.result().toHex() + ".db"));

      QCryptographicHash hash2(QCryptographicHash::Sha1);
      hash2.addData(newName.toUtf8().constData());

      if(oldDb.exists()) {
           oldDb.rename(QString(UTIL::CONFIGDIR + "/" + hash2.result().toHex() + ".db"));
      }
    }
}


//! ----------- on_addPathButton_clicked ---------------------------------------
void PageLibrary::on_addPathButton_clicked()
{
    QString dirName = QFileDialog::getExistingDirectory(m_parent,
                      tr("Add directory"),
                      QString(),
                      QFileDialog::ShowDirsOnly);

    if (!dirName.isEmpty()) {
      ui_paths_list->addItem(dirName);

      QString dbName = ui_choose_db->currentText();
      if (!m_db_params.contains(dbName) || dbName.isEmpty())
        return;

      m_db_params[dbName].sourcePathList.clear();
      for (int i = 0; i < ui_paths_list->count(); i++)
        m_db_params[dbName].sourcePathList << ui_paths_list->item(i)->text();

      _isChanged = true;
    }

}

//! ----------- on_removePathButton_clicked ------------------------------------
void PageLibrary::on_removePathButton_clicked()
{
    foreach (QListWidgetItem *item, ui_paths_list->selectedItems()) {
      delete ui_paths_list->takeItem(ui_paths_list->row(item));
    }

    QString dbName = ui_choose_db->currentText();
    if (!m_db_params.contains(dbName) || dbName.isEmpty()) return;

    m_db_params[dbName].sourcePathList.clear();
    for (int i = 0; i < ui_paths_list->count(); i++)
      m_db_params[dbName].sourcePathList << ui_paths_list->item(i)->text();

    _isChanged = true;
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------
void PageLibrary::on_removeAllPathsButton_clicked()
{
    ui_paths_list->clear();

    QString dbName = ui_choose_db->currentText();
    if (!m_db_params.contains(dbName) || dbName.isEmpty()) return;
    m_db_params[dbName].sourcePathList.clear();

    _isChanged = true;
}


/*
********************************************************************************
*                                                                              *
*    Class PageRadio                                                           *
*                                                                              *
********************************************************************************
*/
PageRadio::PageRadio(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    //! create Gui
    createGui();

    //! connection
    connect(this->addFileButton, SIGNAL(clicked()), this, SLOT(on_addFileButton_clicked()));
    connect(this->removeFileButton, SIGNAL(clicked()), this, SLOT(on_removeFileButton_clicked()));
    connect(this->removeAllFileButton, SIGNAL(clicked()), this, SLOT(on_removeAllFilesButton_clicked()));
}


//! ----------- createGui ------------------------------------------------------
void PageRadio::createGui()
{
    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Radio settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    /*-------------------------------------------------*/
    /* Radio settings widget                           */
    /* ------------------------------------------------*/
    QVBoxLayout* verticalLayout = new QVBoxLayout(main_widget);

    RadioFileList = new QListWidget(main_widget);
    RadioFileList->setMinimumHeight(180);

    QHBoxLayout* hl1 = new QHBoxLayout();

    //! Horizontal Layout
    QLabel *lbl1 = new QLabel(tr("Radio files"), main_widget);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));

    //! Vertical layout for button Remove/Remove All/Add
    addFileButton = new QPushButton(main_widget);
    addFileButton->setText(tr("Add ..."));

    removeFileButton = new QPushButton(main_widget);
    removeFileButton->setText(tr("Remove"));

    removeAllFileButton = new QPushButton(main_widget);
    removeAllFileButton->setText(tr("Remove all"));

    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(addFileButton);
    vl2->addWidget(removeFileButton);
    vl2->addWidget(removeAllFileButton);
    vl2->addItem(verticalSpacer_2);

    hl1->addWidget(RadioFileList);
    hl1->addLayout(vl2);

    verticalLayout->addWidget(lbl1);
    verticalLayout->addLayout(hl1);


    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );


    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageRadio::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    doLayout();
}

void PageRadio::doLayout()
{
    //Debug::debug() << "## PageRadio::doLayout";
    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}


QSize PageRadio::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height() + 50
      );
}

//! ----------- saveSettings ---------------------------------------------------
void PageRadio::saveSettings()
{
    Debug::debug() << "PageRadio::saveSettings";

    //! nothing done yet
    // a voir si il faut définir la liste des radios utilisées (par db)
}

//! ----------- restoreSettings ------------------------------------------------
void PageRadio::restoreSettings()
{
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(UTIL::CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);

    QStringList entryList = sourceDir.entryList(filters, QDir::Files);
    RadioFileList->clear();
    RadioFileList->addItems(entryList);

    _isChanged = false;
}

//! ----------- on_addPathButton_clicked ---------------------------------------
void PageRadio::on_addFileButton_clicked()
{
    QStringList files = QFileDialog::getOpenFileNames(
       m_parent,
       tr("Add xspf radio files"),
       QDesktopServices::storageLocation(QDesktopServices::MusicLocation),
       tr("Radio files(*.xspf);;All files(*.*)"));

    if(files.isEmpty()) return;
    QString fileName = files.takeFirst();
    if (!fileName.isEmpty())
    {
      QFile file(fileName);
      if( file.copy(UTIL::CONFIGDIR + "/radio/" + QFileInfo(fileName).fileName()) ) {
        //! File copy is OK
        RadioFileList->addItem(QFileInfo(fileName).fileName());
        _isChanged = true;
      }
    }

}

//! ----------- on_removePathButton_clicked ------------------------------------
void PageRadio::on_removeFileButton_clicked()
{
    foreach (QListWidgetItem *item, RadioFileList->selectedItems()) {
      QString filename =  item->data(Qt::DisplayRole).toString();
      Debug::debug() << "SettingRadioPage -  on_removeFileButton_clicked " << filename;
      QFile file(UTIL::CONFIGDIR + "/radio/" + filename);

      if(file.exists()) {
        file.remove();
        delete RadioFileList->takeItem(RadioFileList->row(item));
        _isChanged = true;
      }
    }
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------
void PageRadio::on_removeAllFilesButton_clicked()
{
    RadioFileList->clear();
    _isChanged = true;
}


/*
********************************************************************************
*                                                                              *
*    Class PageShortcut                                                        *
*                                                                              *
********************************************************************************
*/
PageShortcut::PageShortcut(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    createGui();

}

PageShortcut::~PageShortcut()
{

}

void PageShortcut::createGui()
{
    //Debug::debug() << "###### PageShortcut createGui";

    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Shortcut settings");
    m_title->m_width  = m_parent->width();

    m_items["play"]        = new ShortcutGraphicItem(tr("Play/Pause"), SETTINGS()->_shortcutsKey["play"], QPixmap(":/images/media-play.png"));
    m_items["stop"]        = new ShortcutGraphicItem(tr("Stop"), SETTINGS()->_shortcutsKey["stop"], QPixmap(":/images/media-stop.png"));
    m_items["prev_track"]  = new ShortcutGraphicItem(tr("Previous track"), SETTINGS()->_shortcutsKey["prev_track"], QPixmap(":/images/media-prev.png"));
    m_items["next_track"]  = new ShortcutGraphicItem(tr("Next track"), SETTINGS()->_shortcutsKey["next_track"], QPixmap(":/images/media-next.png"));
    m_items["inc_volume"]  = new ShortcutGraphicItem(tr("Increase volume"), SETTINGS()->_shortcutsKey["inc_volume"], QPixmap(":/images/volume-icon.png"));
    m_items["dec_volume"]  = new ShortcutGraphicItem(tr("Decrease volume"), SETTINGS()->_shortcutsKey["dec_volume"], QPixmap(":/images/volume-icon.png"));
    m_items["mute_volume"] = new ShortcutGraphicItem(tr("Mute/Unmute volume"), SETTINGS()->_shortcutsKey["mute_volume"], QPixmap(":/images/volume-muted.png"));
    m_items["jump_to_track"] = new ShortcutGraphicItem(tr("Jump to track"), SETTINGS()->_shortcutsKey["jump_to_track"], QPixmap(":/images/goto-32x32.png"));

    connect(m_items.value("play"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("stop"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("prev_track"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("next_track"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("inc_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("dec_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("mute_volume"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));
    connect(m_items.value("jump_to_track"), SIGNAL(clicked()),this, SLOT(shorcutItemClicked()));

    m_items.value("play")->setParentItem(this);
    m_items.value("stop")->setParentItem(this);
    m_items.value("prev_track")->setParentItem(this);
    m_items.value("next_track")->setParentItem(this);
    m_items.value("inc_volume")->setParentItem(this);
    m_items.value("dec_volume")->setParentItem(this);
    m_items.value("mute_volume")->setParentItem(this);
    m_items.value("jump_to_track")->setParentItem(this);

    m_title->setParentItem(this);
    m_title->setPos(5,0);
}

void PageShortcut::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    //Debug::debug() << "## PageShortcut::resizeEvent";
    doLayout();
}

void PageShortcut::doLayout()
{
    //Debug::debug() << "## PageShortcut::doLayout";
    m_title->m_width =  m_parent->width();

    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    int Ypos = 50;

    m_items.value("play")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("stop")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("prev_track")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("next_track")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("inc_volume")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("dec_volume")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("mute_volume")->setPos(Xpos,Ypos);
    Ypos += 35;
    m_items.value("jump_to_track")->setPos(Xpos,Ypos);
}


QSize PageShortcut::size()
{
    return QSize(
      m_parent->width()-350,
      7*50
      );
}


//! ----------- restoreSettings ------------------------------------------------
void PageShortcut::restoreSettings()
{
    Debug::debug() << "PageShortcut restoreSettings";
    m_items["play"]->m_key          = SETTINGS()->_shortcutsKey["play"];
    m_items["stop"]->m_key          = SETTINGS()->_shortcutsKey["stop"];
    m_items["prev_track"]->m_key    = SETTINGS()->_shortcutsKey["prev_track"];
    m_items["next_track"]->m_key    = SETTINGS()->_shortcutsKey["next_track"];
    m_items["inc_volume"]->m_key    = SETTINGS()->_shortcutsKey["inc_volume"];
    m_items["dec_volume"]->m_key    = SETTINGS()->_shortcutsKey["dec_volume"];
    m_items["mute_volume"]->m_key   = SETTINGS()->_shortcutsKey["mute_volume"];
    m_items["jump_to_track"]->m_key = SETTINGS()->_shortcutsKey["jump_to_track"];


    m_items["play"]->m_status = GlobalShortcuts::instance()->shortcuts().value("play").status;
    m_items["stop"]->m_status = GlobalShortcuts::instance()->shortcuts().value("stop").status;
    m_items["prev_track"]->m_status = GlobalShortcuts::instance()->shortcuts().value("prev_track").status;
    m_items["next_track"]->m_status = GlobalShortcuts::instance()->shortcuts().value("next_track").status;
    m_items["inc_volume"]->m_status = GlobalShortcuts::instance()->shortcuts().value("inc_volume").status;
    m_items["dec_volume"]->m_status = GlobalShortcuts::instance()->shortcuts().value("dec_volume").status;
    m_items["mute_volume"]->m_status = GlobalShortcuts::instance()->shortcuts().value("mute_volume").status;
    m_items["jump_to_track"]->m_status = GlobalShortcuts::instance()->shortcuts().value("jump_to_track").status;

    _isChanged = false;
    update();
}

//! ----------- saveSettings ------------------------------------------------
void PageShortcut::saveSettings()
{
    Debug::debug() << "PageShortcut::saveSettings";

    SETTINGS()->_shortcutsKey["play"]          = m_items["play"]->m_key;
    SETTINGS()->_shortcutsKey["stop"]          = m_items["stop"]->m_key;
    SETTINGS()->_shortcutsKey["prev_track"]    = m_items["prev_track"]->m_key;
    SETTINGS()->_shortcutsKey["next_track"]    = m_items["next_track"]->m_key;
    SETTINGS()->_shortcutsKey["inc_volume"]    = m_items["inc_volume"]->m_key;
    SETTINGS()->_shortcutsKey["dec_volume"]    = m_items["dec_volume"]->m_key;
    SETTINGS()->_shortcutsKey["mute_volume"]   = m_items["mute_volume"]->m_key;
    SETTINGS()->_shortcutsKey["jump_to_track"] = m_items["jump_to_track"]->m_key;
}

void PageShortcut::shorcutItemClicked()
{
    //Debug::debug() << "PageShortcut shorcutItemClicked";

    ShortcutGraphicItem *item = qobject_cast<ShortcutGraphicItem *>(sender());
    if(!item) return;

    QString item_name = m_items.key(item);
    QString shortcut  = item->m_key;

    ShortcutDialog* dialog = new ShortcutDialog(item_name, shortcut, m_parent);


    if(dialog->exec() == QDialog::Accepted)
    {
      item->m_key = dialog->newShortcut();
      item->update();
     _isChanged = true;
    }

    delete dialog;
}

/*
********************************************************************************
*                                                                              *
*    Class ShortcutGraphicItem                                                 *
*                                                                              *
********************************************************************************
*/
ShortcutGraphicItem::ShortcutGraphicItem(const QString& name, const QString& key, QPixmap pix)
{
    m_name   = name;
    m_key    = key;
    m_pixmap = pix.scaled(QSize(24,24), Qt::KeepAspectRatio, Qt::SmoothTransformation);
    m_status = true;

    setAcceptsHoverEvents(true);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
    setCursor(Qt::PointingHandCursor);

    setGraphicsItem(this);
    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    //! option configuration
    QStyle *style =  QApplication::style();
    opt.widget = 0;
    opt.palette = QApplication::palette();
    opt.font = QApplication::font();
    opt.fontMetrics = QFontMetrics(opt.font);

    opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
    int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);

    opt.decorationSize = QSize(pm, pm);
    opt.decorationPosition = QStyleOptionViewItem::Top;
    opt.displayAlignment = Qt::AlignCenter;

    opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
    opt.state |= QStyle::State_Active;
    opt.state |= QStyle::State_Enabled;
    opt.state &= ~QStyle::State_Selected;
}

QRectF ShortcutGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 450, 30);
}

// Inherited from QGraphicsLayoutItem
void ShortcutGraphicItem::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF ShortcutGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}

void ShortcutGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect();
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);

   //! paint shortcut name
   painter->setPen(QApplication::palette().color(QPalette::Normal,QPalette::WindowText)) ;
   painter->setFont(opt.font);

   painter->drawPixmap(4, 3, m_pixmap);
   painter->drawText(QRect (32,0,300, 30), Qt::AlignLeft | Qt::AlignVCenter,m_name );
   painter->drawText(QRect (310,0,190, 30), Qt::AlignLeft | Qt::AlignVCenter,m_key );

   if(m_status == false)
     painter->drawPixmap(400, 5, QPixmap(":/images/media-broken-18x18.png") );
   else
     painter->drawPixmap(400, 5, 20,20, QPixmap(":/images/checkmark-48x48.png") );
}

void ShortcutGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
    opt.state |= QStyle::State_MouseOver;
    this->update();
}

void ShortcutGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
    opt.state &= ~QStyle::State_MouseOver;
    this->update();
}

void ShortcutGraphicItem::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
Q_UNUSED(event)
    emit clicked();
}



/*
********************************************************************************
*                                                                              *
*    Class PageScrobbler                                                       *
*                                                                              *
********************************************************************************
*/
PageScrobbler::PageScrobbler(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    m_title           = new TitleGraphicItem();
    m_title->m_name   = tr("Scrobbler settings");
    m_title->m_width  = m_parent->width();

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    /*-------------------------------------------------*/
    /* Scrobbler settings widget                       */
    /* ------------------------------------------------*/
    QVBoxLayout* vl0 = new QVBoxLayout(main_widget);

    //! Check box for lastFm scrobbler enable/disable
    useLastFmScrobbler = new QCheckBox(main_widget);
    useLastFmScrobbler->setText(tr("Use LastFm scrobbler"));

    //! lastfm connexion
    QHBoxLayout* hl0 = new QHBoxLayout();
    statusLabel  = new QLabel(main_widget);
    statusPixmap = new QLabel(main_widget);
    hl0->addWidget(statusPixmap);
    hl0->addWidget(statusLabel);
    hl0->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));

    accountGroupB = new QGroupBox(tr("Account details"), main_widget);
    accountGroupB->setFlat(true);

    QGridLayout* gl = new QGridLayout(accountGroupB);
    gl->setContentsMargins(0, 0, 0, 0);

    QLabel* label_1 = new QLabel(tr("Last.fm username"), accountGroupB);
    gl->addWidget(label_1, 0, 0, 1, 1);

    QLabel* label_2 = new QLabel(tr("Last.fm password"), accountGroupB);
    gl->addWidget(label_2, 1, 0, 1, 1);

    lineEdit_1 = new QLineEdit(accountGroupB);
    lineEdit_1->setMinimumWidth(QFontMetrics(QFont()).width("WWWWWWWWWWWW"));
    gl->addWidget(lineEdit_1, 0, 1, 1, 1);

    lineEdit_2 = new QLineEdit(accountGroupB);
    lineEdit_2->setEchoMode(QLineEdit::Password);
    gl->addWidget(lineEdit_2, 1, 1, 1, 1);

    //! sign in button
    signInButton = new QPushButton(main_widget);
    signInButton->setText(tr("Sign In"));

    //! connextion status
    vl0->addWidget(useLastFmScrobbler);
    vl0->addLayout(hl0);
    vl0->addWidget(accountGroupB);
    vl0->addWidget(signInButton);
    vl0->addItem(new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding));

    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

    m_title->setParentItem(this);
    m_title->setPos(5,0);


    // scrobbler & slots
    scrobbler  = new LastFm::Scrobbler();

    connect(this->useLastFmScrobbler, SIGNAL(clicked()), this, SLOT(enableChange()));
    connect(this->signInButton, SIGNAL(clicked()), this, SLOT(signInClicked()));
    connect(scrobbler, SIGNAL(signInFinished()), this, SLOT(slotSignInDone()));
}

void PageScrobbler::resizeEvent( QGraphicsSceneResizeEvent *event )
{
Q_UNUSED(event)
    doLayout();
}

void PageScrobbler::doLayout()
{
    //Debug::debug() << "## PageScrobbler::doLayout";

    m_title->m_width =  m_parent->width();
    int Xpos = m_parent->width() - 350 > 0 ? (m_parent->width() - 350)/2 : 50;
    proxy_widget->setPos(Xpos,50);
}

QSize PageScrobbler::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height() + 50
      );
}

//! ----------- saveSettings ---------------------------------------------------
void PageScrobbler::saveSettings()
{
    Debug::debug() << "PageScrobbler::saveSettings";

    SETTINGS()->_useLastFmScrobbler = this->useLastFmScrobbler->isChecked();
}

//! ----------- restoreSettings ------------------------------------------------
void PageScrobbler::restoreSettings()
{
    _isEnableOld = SETTINGS()->_useLastFmScrobbler;
    this->useLastFmScrobbler->setChecked(_isEnableOld);

    enableChange();

    updateSignInStatus();

    _isChanged = false;
}


//! ----------- enableChange ---------------------------------------------------
void PageScrobbler::enableChange()
{
    //Debug::debug() << "SettingScrobblerPage::enableChange";
    bool isEnableNew = this->useLastFmScrobbler->isChecked();

    if(isEnableNew != _isEnableOld)
       _isChanged = true;
    else
       _isChanged = false;

    // update state according to enable status
    statusLabel->setEnabled(isEnableNew);
    signInButton->setEnabled(isEnableNew);
    accountGroupB->setEnabled(isEnableNew);
}


//! ----------- getAuthentificationStatus --------------------------------------
void PageScrobbler::updateSignInStatus()
{
    bool auth = scrobbler->isAuthenticated();
    if(auth) {
      lineEdit_1->setText(scrobbler->username());
      lineEdit_2->clear();
      statusPixmap->setPixmap(QPixmap(":/images/checkmark-48x48.png").scaled(QSize(24,24)));
      //statusPixmap->setPixmap(QPixmap(":/images/signal_accepted-48x48.png").scaled(QSize(24,24)));
      statusLabel->setText(QString(tr("You are log in lastFm service as <b>%1</b>").arg(scrobbler->username())));

      signInButton->setText(tr("Sign Out"));
    }
    else {
      statusPixmap->setPixmap(QPixmap(":/images/warning-48x48.png").scaled(QSize(24,24)));
      statusLabel->setText(tr("You are not log into"));
      signInButton->setText(tr("Sign In"));
    }
}


//! ----------- signInClicked --------------------------------------------------
void PageScrobbler::signInClicked()
{
    _isChanged = true;
    bool auth = scrobbler->isAuthenticated();
    if(auth) {
       lineEdit_1->clear();
       lineEdit_2->clear();
       scrobbler->signOut();
       updateSignInStatus();
    }
    else {
      scrobbler->signIn(lineEdit_1->text(), lineEdit_2->text());
    }
}



//! ----------- slotSignInDone -------------------------------------------------
void PageScrobbler::slotSignInDone()
{
    bool success = scrobbler->isAuthenticated();
    updateSignInStatus();

    if (!success)
      QMessageBox::warning(m_parent, tr("Authentication failed"), tr("Your Last.fm credentials were incorrect"));
}



/*
********************************************************************************
*                                                                              *
*    Class ControlWidget                                                       *
*                                                                              *
********************************************************************************
*/
ControlWidget::ControlWidget(QWidget* parentView) : QGraphicsWidget(0)
{
    m_parent       = parentView;

    QPushButton  *save_button   = new QPushButton(QIcon(":/images/save-32x32.png"), tr("Apply"), 0);
    QPushButton  *cancel_button = new QPushButton(QIcon::fromTheme("dialog-cancel"), tr("Cancel"), 0);

    connect(save_button, SIGNAL(clicked()), this, SIGNAL(save_clicked()));
    connect(cancel_button, SIGNAL(clicked()), this, SIGNAL(cancel_clicked()));

    // main widget
    QWidget* main_widget = new QWidget();

    main_widget->setAttribute(Qt::WA_NoBackground, true);
    main_widget->setAutoFillBackground(true);

    QHBoxLayout *hbox = new QHBoxLayout(main_widget);
    hbox->addWidget(save_button);
    hbox->addWidget(cancel_button);

    // proxy widget
    proxy_widget = new QGraphicsProxyWidget( this );
    proxy_widget->setWidget( main_widget );
    proxy_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );


}


QSize ControlWidget::size()
{
    return QSize(
      proxy_widget->geometry().size().width(),
      proxy_widget->geometry().size().height()
      );
}
