/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/
#ifndef _SETTINGS_WIDGETS_H_
#define _SETTINGS_WIDGETS_H_

// local
#include "databasemanager.h"
#include "scrobblerlastfm.h"
#include "shortcutdialog.h"

// Qt
#include <QGraphicsWidget>
#include <QGraphicsLinearLayout>
#include <QtGui>

class TitleGraphicItem;



/*
********************************************************************************
*                                                                              *
*    Class PageGeneral                                                         *
*                                                                              *
********************************************************************************
*/
class PageGeneral : public QGraphicsWidget
{
Q_OBJECT
  public:
    PageGeneral(QWidget* parentView = 0);

    void restoreSettings();
    void saveSettings();

    bool isSystrayChanged();
    bool isDbusChanged();
    bool isMprisChanged();

    void doLayout();
    QSize size();

  private:
    void createGui();

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private:
    QCheckBox            *ui_check_systray;
    QCheckBox            *ui_check_hideAtStartup;
    QCheckBox            *ui_check_vol_notif;
    QCheckBox            *ui_check_enableMpris;
    QCheckBox            *ui_check_enable_mouseover;
    QGroupBox            *ui_groupbox_dbus;

    TitleGraphicItem     *m_title;
    QGraphicsProxyWidget *proxy_widget;
    QWidget              *m_parent;
};


/*
********************************************************************************
*                                                                              *
*    Class PagePlayer                                                          *
*                                                                              *
********************************************************************************
*/
class PagePlayer : public QGraphicsWidget
{
Q_OBJECT
  public:
    PagePlayer(QWidget* parentView = 0);

    void restoreSettings();
    void saveSettings();

    void doLayout();
    QSize size();

  private:
    void createGui();

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private:
    QCheckBox             *ui_stopOnPlayqueueClear;
    QCheckBox             *ui_restartPlayingAtStartup;

    QGroupBox             *ui_groupBox;
    QRadioButton          *ui_mode_track;
    QRadioButton          *ui_mode_album;

    TitleGraphicItem      *m_title;
    QGraphicsProxyWidget  *proxy_widget;
    QWidget               *m_parent;
};



/*
********************************************************************************
*                                                                              *
*    Class PageLibrary                                                         *
*                                                                              *
********************************************************************************
*/
class PageLibrary : public QGraphicsWidget
{
Q_OBJECT
  public:
    PageLibrary(QWidget* parentView = 0);

    void restoreSettings();
    void saveSettings();

    bool isChanged() {return _isChanged;}

    void doLayout();
    QSize size();

  private:
    void createGui();
    void addDatabaseParam(const QString& name, const DB::S_dbParam& dbParam);

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private slots:
    void loadDatabaseParam(QString);
    void newDatabaseParam();
    void delDatabaseParam();
    void updateDatabaseParam1();
    void updateDatabaseParam2();
    void renameDatabaseParam();
    void enableMultiDatabase();
    void on_addPathButton_clicked();
    void on_removePathButton_clicked();
    void on_removeAllPathsButton_clicked();

  private:
    bool         _isChanged; //! we need to rebuild database

    QListWidget  *ui_paths_list;
    QCheckBox    *ui_enable_multiDb;
    QCheckBox    *ui_auto_update;
    QCheckBox    *ui_search_cover;
    QComboBox    *ui_choose_db;

    QPushButton  *ui_add_path_button;
    QPushButton  *ui_remove_path_button;
    QPushButton  *ui_remove_all_button;

    QToolButton  *ui_db_new_button;
    QToolButton  *ui_db_del_button;
    QToolButton  *ui_db_rename_button;

    QMap<QString, DB::S_dbParam> m_db_params;

    TitleGraphicItem      *m_title;
    QGraphicsProxyWidget  *proxy_widget;
    QWidget               *m_parent;
};


/*
********************************************************************************
*                                                                              *
*    Class PageRadio                                                           *
*                                                                              *
********************************************************************************
*/
class PageRadio : public QGraphicsWidget
{
Q_OBJECT
  public:
    PageRadio(QWidget* parentView = 0);

    void restoreSettings();
    void saveSettings();

    bool isChanged() {return _isChanged;}

    void doLayout();
    QSize size();

  private:
    void createGui();

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private slots:
    void on_addFileButton_clicked();
    void on_removeFileButton_clicked();
    void on_removeAllFilesButton_clicked();

  private:
    bool            _isChanged;
    QListWidget     *RadioFileList;
    QPushButton     *addFileButton;
    QPushButton     *removeFileButton;
    QPushButton     *removeAllFileButton;

    TitleGraphicItem      *m_title;
    QGraphicsProxyWidget  *proxy_widget;
    QWidget               *m_parent;
};



/*
********************************************************************************
*                                                                              *
*    Class ShortcutGraphicItem                                                 *
*                                                                              *
********************************************************************************
*/
class ShortcutGraphicItem : public QGraphicsObject, public QGraphicsLayoutItem
{
Q_OBJECT
  public:
    ShortcutGraphicItem(const QString&, const QString&, QPixmap);

    QString m_name;
    QString m_key;
    QPixmap m_pixmap;
    bool    m_status;

  protected:
    // Inherited from QGraphicsLayoutItem
    void setGeometry(const QRectF &geom);
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;

    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent ( QGraphicsSceneMouseEvent * event );

  private:
    QStyleOptionViewItemV4 opt;

  signals:
    void clicked();
};

/*
********************************************************************************
*                                                                              *
*    Class PageShortcut                                                        *
*                                                                              *
********************************************************************************
*/
class PageShortcut : public QGraphicsWidget
{
Q_OBJECT
  public:
    PageShortcut(QWidget* parentView = 0);
    ~PageShortcut();

    void restoreSettings();
    void saveSettings();

    bool isChanged() {return _isChanged;}

    void doLayout();
    QSize size();

  private:
    void createGui();

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private slots:
    void shorcutItemClicked();

  private:
    bool                  _isChanged;

    QMap<QString, ShortcutGraphicItem*>  m_items;
    QWidget               *m_parent;

    TitleGraphicItem      *m_title;
    //QCheckBox             *ui_enable_shortcut;
    QGraphicsProxyWidget  *proxy_widget;
};


/*
********************************************************************************
*                                                                              *
*    Class PageScrobbler                                                       *
*                                                                              *
********************************************************************************
*/
class PageScrobbler : public QGraphicsWidget
{
Q_OBJECT
  public:
    PageScrobbler(QWidget* parentView = 0);

    void restoreSettings();
    void saveSettings();

    bool isChanged() {return _isChanged;}

    void doLayout();
    QSize size();

  private slots:
    void signInClicked();
    void slotSignInDone();

  private:
    void updateSignInStatus();

  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );

  private:
    TitleGraphicItem      *m_title;
    QGraphicsProxyWidget  *proxy_widget;
    QWidget               *m_parent;

    QCheckBox    *useLastFmScrobbler;
    QGroupBox    *accountGroupB;
    QLineEdit    *lineEdit_1;
    QLineEdit    *lineEdit_2;
    QPushButton  *signInButton;
    QLabel       *statusLabel;
    QLabel       *statusPixmap;

    bool         _isChanged;
    bool         _isEnableOld;
    LastFm::Scrobbler *scrobbler;

  private slots:
    void enableChange();
 };

/*
********************************************************************************
*                                                                              *
*    Class ControlWidget                                                       *
*                                                                              *
********************************************************************************
*/
class ControlWidget : public QGraphicsWidget
{
Q_OBJECT
  public:
    ControlWidget(QWidget* parentView = 0);
    QSize size();

  private:
    QGraphicsProxyWidget  *proxy_widget;
    QWidget               *m_parent;

    QPushButton  *save_button;
    QPushButton  *cancel_button;
signals:
    void save_clicked();
    void cancel_clicked();
 };

#endif // _SETTINGS_WIDGETS_H_
