/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#ifndef _STREAM_SCENE_H_
#define _STREAM_SCENE_H_

#include "StreamModel/service_base.h"
#include "CollectionView/graphicitem_menu.h"
#include "EnginePlayer/engineplayer.h"
#include "viewmode.h"


#include <QStringList>
#include <QGraphicsScene>
#include <QGraphicsItem>
#include <QtGui>

class VirtualPlayqueue;
class StreamModel;

/*
********************************************************************************
*                                                                              *
*    Class StreamScene                                                         *
*                                                                              *
********************************************************************************
*/
class StreamScene : public QGraphicsScene
{
Q_OBJECT
  public:
    StreamScene(QWidget *parent);
    ~StreamScene();
    int   elementCount() const {return m_infoSize;}

    MAIN::ViewMode            m_view_mode;

    //! playing item scene method (for Mainwindow access)
    void playSelected();

  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );

  private slots :
    void slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU);
    void slot_item_mouseMove();

    void slot_streams_fetched(MEDIA::TrackPtr);
    void slot_reload_stream_list();
    void slot_remove_duplicate_stream();
    void slot_edit_stream_list();

  public slots:
    void populateScene();

  private:
    //! selection
    QList<QGraphicsItem*> sortedSelectedItem();
    void updateStreamFavorite();

    //! population scene method
    void populateStreamScene();
    void populateExtendedStreamScene();
    void populateWaitingScene(QString text);

    //! playing item scene method
    void playStream();

    //! playqueue method
    void enqueueStream(bool is_new_playqueue);
    void enqueueSelected(bool is_new_playqueue);
    void startStreamsDrag(QGraphicsItem* i=0);
    
signals:
    void repopulateStreamScene();
    
  private :
    /*  stream services */
    QMap<MAIN::ViewMode,Service*>   m_services;
      
    StreamModel              *m_model;

    EnginePlayer             *_player;
    int                       m_infoSize;
    QWidget                  *_parentWidget;

    QGraphicsItem            *m_mouseGrabbedItem;

    QMap<QString,QAction*>    m_actions;

    GraphicsItemMenu         *m_graphic_item_menu;
    
    //! messages Id for StatusWidget management
    QMap<QString, uint>       m_messageIds;
};

#endif // _STREAM_SCENE_H_
